package geniusweb.party;

import java.util.Set;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import geniusweb.profile.Profile;

/**
 * The capabilities of a party
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
public class Capabilities {
	/**
	 * List of supported protocol names
	 */
	private final Set<String> behaviours;
	/**
	 * The profile classes that are supported
	 */
	private final Set<Class<? extends Profile>> profiles;

	/**
	 * 
	 * @param behaviours the protocols that a Party can handle
	 */
	@JsonCreator
	public Capabilities(@JsonProperty("behaviours") Set<String> behaviours,
			@JsonProperty("profiles") Set<Class<? extends Profile>> profiles) {
		if (behaviours == null) {
			throw new IllegalArgumentException("behaviours==null");
		}
		for (Class<? extends Profile> prof : profiles) {
			if (!Profile.class.isAssignableFrom(prof))
				throw new IllegalArgumentException(
						"profile " + prof + " must be a subclass of Profile");
		}
		this.behaviours = behaviours;
		this.profiles = profiles;
	}

	/**
	 * 
	 * @return the behaviours (protocols) that are supported
	 */
	public Set<String> getBehaviours() {
		return behaviours;
	}

	/**
	 * 
	 * @return the profile classes that are supported
	 * 
	 */
	public Set<Class<? extends Profile>> getProfiles() {
		return profiles;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((behaviours == null) ? 0 : behaviours.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Capabilities other = (Capabilities) obj;
		if (behaviours == null) {
			if (other.behaviours != null)
				return false;
		} else if (!behaviours.equals(other.behaviours))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Capabilities[" + "Behaviours=" + behaviours + "]";
	}

}
